package uk.co.agileworks.tickets;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import org.openqa.selenium.By;
import org.openqa.selenium.NoSuchElementException;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.htmlunit.HtmlUnitDriver;
import org.openqa.selenium.support.ui.Select;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

@Service
public final class EventSearcherImpl implements EventSearcher {

	private static final Logger LOGGER = LoggerFactory.getLogger(EventSearcherImpl.class);

	private static final boolean ENABLE_JAVASCRIPT = false;
	private static final String EVENT_SELECT_ID = "sport";

	@Resource
	private String ticketUrl;
	private WebDriver driver;

	@Override
	public List<String> getEvents() {
		init();
		List<String> events = new ArrayList<String>();
		for (WebElement option : findSelect(EVENT_SELECT_ID).getOptions()) {
			if (!"Select a sport".equals(option.getText())) {
				events.add(option.getText());
			}
		}
		LOGGER.debug("got events=" + events);
		return events;
	}

	@Override
	public boolean hasTickets(String event) {
		init();
		boolean hasTickets = false;
		findSelect(EVENT_SELECT_ID).selectByVisibleText(event);
		driver.findElement((By.xpath("//input[@value='Search']"))).click();
		debug();
		try {
			driver.findElement(By.xpath("//div[@class='mgBot10 searchPagi']"));
			hasTickets = true;
		} catch (NoSuchElementException ignore) {
		}
		return hasTickets;
	}

	void setTicketUrl(String ticketUrl) {
		this.ticketUrl = ticketUrl;
	}

	private void debug() {
		LOGGER.debug("current url=" + driver.getCurrentUrl());
		LOGGER.debug("page title=" + driver.getTitle());
		LOGGER.debug("page source...\n" + driver.getPageSource());
	}

	private Select findSelect(String id) {
		return new Select(driver.findElement(By.id(id)));
	}

	private void init() {
		driver = new HtmlUnitDriver(ENABLE_JAVASCRIPT);
		driver.get(ticketUrl);
		debug();
	}

}
