package uk.co.agileworks.tickets;

import static junit.framework.Assert.assertFalse;
import static junit.framework.Assert.assertTrue;

import java.util.List;

import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.AbstractJUnit4SpringContextTests;

@ContextConfiguration(locations = { "classpath:testsite-tickets-test.xml" })
public final class EventSearcherTest extends AbstractJUnit4SpringContextTests {

	private static final Logger LOGGER = LoggerFactory.getLogger(EventSearcherTest.class);

	private static final String SEARCH_ATHLETICS = "http://localhost:7080/search-athletics.html";
	private static final String SEARCH_OLYMPIC_PARK = "http://localhost:7080/search-olympicPark.html";

	@Autowired
	private EventSearcherImpl eventSearcher;

	@Test
	public void getEvents() {
		init(SEARCH_ATHLETICS);
		List<String> events = eventSearcher.getEvents();
		LOGGER.trace("events=" + events);
		assertTrue(events.contains("Athletics"));
		assertTrue(events.contains("Olympic Park"));
		assertFalse(events.contains("Select a sport"));
	}

	@Test
	public void hasTickets() {
		init(SEARCH_OLYMPIC_PARK);
		boolean hasTickets = eventSearcher.hasTickets("Olympic Park");
		LOGGER.trace("Olympic park=" + hasTickets);
		assertTrue(hasTickets);
	}

	@Test
	public void noTickets() {
		init(SEARCH_ATHLETICS);
		boolean hasTickets = eventSearcher.hasTickets("Athletics");
		LOGGER.trace("Athletics=" + hasTickets);
		assertFalse(hasTickets);
	}

	private void init(String url) {
		eventSearcher.setTicketUrl(url);
	}

}
