package uk.co.agileworks.testsite.service;

import static uk.co.agileworks.testsite.test.SpringTest.assertEquals;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;

import org.apache.cxf.interceptor.Interceptor;
import org.apache.cxf.jaxws.JaxWsProxyFactoryBean;
import org.apache.cxf.message.Message;
import org.apache.cxf.ws.security.wss4j.WSS4JOutInterceptor;
import org.apache.ws.security.WSConstants;
import org.apache.ws.security.WSPasswordCallback;
import org.apache.ws.security.handler.WSHandlerConstants;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.webapp.WebAppContext;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import uk.co.agileworks.testsite.domain.Question;
import uk.co.agileworks.testsite.test.DataSet;

public final class WebServiceTest {

	private static final int PORT = 7080;
	private static final String USERNAME = "testsite";
	private static final String PASSWORD = "password";
	private static final Question QUESTION_1 = new DataSet().getQuestion();
	
	private static Server server;

	@BeforeClass
	public static void start() {
		server = new Server(PORT);
		WebAppContext context = new WebAppContext();
		context.setContextPath("/testsite-web");
		context.setWar("target/testsite-web");
		server.setHandler(context);
		try {
			server.start();
		} catch (Exception e) {
			throw new IllegalStateException("Jetty didn't start.");
		}
	}
	
	@AfterClass
	public static void stop() {
		try {
			server.stop();
		} catch (Exception e) {
			throw new IllegalStateException("Jetty didn't stop.");
		}
	}

	@Test
	public void getQuestion() {
		assertEquals(QUESTION_1, getService().getQuestion(1L));
	}

	private QuestionService getService() {
		JaxWsProxyFactoryBean factory = new JaxWsProxyFactoryBean();
		factory.setAddress("http://localhost:" + PORT + "/testsite-web/services/QuestionService");
		factory.setOutInterceptors(getInterceptors());
		return factory.create(QuestionService.class);
	}
	
	private List<Interceptor<? extends Message>> getInterceptors() {
		Map<String, Object> props = new HashMap<String, Object>();
		props.put(WSHandlerConstants.ACTION, WSConstants.USERNAME_TOKEN_LN);
		props.put(WSHandlerConstants.MUST_UNDERSTAND, "false");
		props.put(WSHandlerConstants.PASSWORD_TYPE, WSConstants.PW_DIGEST);
		props.put(WSHandlerConstants.PW_CALLBACK_REF, new ClientPasswordCallback());
		props.put(WSHandlerConstants.USER, USERNAME);
		List<Interceptor<? extends Message>> interceptors = new ArrayList<Interceptor<?>>();
		interceptors.add(new WSS4JOutInterceptor(props));
		return interceptors;
	}
	
	class ClientPasswordCallback implements CallbackHandler {
		@Override
		public void handle(Callback[] callbacks) throws IOException,
				UnsupportedCallbackException {
			WSPasswordCallback pc = (WSPasswordCallback) callbacks[0];
			pc.setPassword(PASSWORD);
		}
	}


}
